@@ -20,8 +20,10 @@ from manifestparser import TestManifest
 from marionette import Marionette
 from mixins.b2g import B2GTestResultMixin, get_b2g_pid, get_dm
 from mozhttpd import MozHttpd
+from mozlog.structured.structuredlog import get_default_logger
 from moztest.adapters.unit import StructuredTestRunner, StructuredTestResult
 from moztest.results import TestResultCollection, TestResult, relevant_line
+import mozversion
 
 class MarionetteTest(TestResult):
 
@@ -45,12 +47,13 @@ class MarionetteTestResult(StructuredTestResult, TestResultCollection):
         self.testsRun = 0
         self.result_modifiers = [] # used by mixins to modify the result
         pid = kwargs.pop('b2g_pid')
+        logcat_stdout = kwargs.pop('logcat_stdout')
         if pid:
             if B2GTestResultMixin not in self.__class__.__bases__:
                 bases = [b for b in self.__class__.__bases__]
                 bases.append(B2GTestResultMixin)
                 self.__class__.__bases__ = tuple(bases)
-            B2GTestResultMixin.__init__(self, b2g_pid=pid)
+            B2GTestResultMixin.__init__(self, b2g_pid=pid, logcat_stdout=logcat_stdout)
         StructuredTestResult.__init__(self, *args, **kwargs)
 
     @property
@@ -202,6 +205,7 @@ class MarionetteTextTestRunner(StructuredTestRunner):
         self.capabilities = kwargs.pop('capabilities')
         self.pre_run_functions = []
         self.b2g_pid = None
+        self.logcat_stdout = kwargs.pop('logcat_stdout')
 
         if self.capabilities["device"] != "desktop" and self.capabilities["browserName"] == "B2G":
             def b2g_pre_run():
@@ -219,7 +223,9 @@ class MarionetteTextTestRunner(StructuredTestRunner):
                                 self.verbosity,
                                 marionette=self.marionette,
                                 b2g_pid=self.b2g_pid,
-                                logger=self.logger)
+                                logger=self.logger,
+                                logcat_stdout=self.logcat_stdout,
+                                result_callbacks=self.result_callbacks)
 
     def run(self, test):
         "Run the given test case or test suite."
@@ -271,6 +277,11 @@ class BaseMarionetteOptions(OptionParser):
                         dest='logdir',
                         action='store',
                         help='directory to store logcat dump files')
+        self.add_option('--logcat-stdout',
+                        action='store_true',
+                        dest='logcat_stdout',
+                        default=False,
+                        help='dump adb logcat to stdout')
         self.add_option('--address',
                         dest='address',
                         action='store',
@@ -279,6 +290,10 @@ class BaseMarionetteOptions(OptionParser):
                         dest='device_serial',
                         action='store',
                         help='serial ID of a device to use for adb / fastboot')
+        self.add_option('--adb-host',
+                        help='host to use for adb connection')
+        self.add_option('--adb-port',
+                        help='port to use for adb connection')
         self.add_option('--type',
                         dest='type',
                         action='store',
@@ -321,7 +336,7 @@ class BaseMarionetteOptions(OptionParser):
                         help='xml output')
         self.add_option('--testvars',
                         dest='testvars',
-                        action='store',
+                        action='append',
                         help='path to a json file with any test data required')
         self.add_option('--tree',
                         dest='tree',
@@ -368,7 +383,8 @@ class BaseMarionetteOptions(OptionParser):
                         help="Define the path to store log file. If the path is"
                              " a directory, the real log file will be created"
                              " given the format gecko-(timestamp).log. If it is"
-                             " a file, if will be used directly. Default: 'gecko.log'")
+                             " a file, if will be used directly. '-' may be passed"
+                             " to write to stdout. Default: './gecko.log'")
         self.add_option('--logger-name',
                         dest='logger_name',
                         action='store',
@@ -437,12 +453,13 @@ class BaseMarionetteTestRunner(object):
     def __init__(self, address=None, emulator=None, emulator_binary=None,
                  emulator_img=None, emulator_res='480x800', homedir=None,
                  app=None, app_args=None, binary=None, profile=None,
-                 logger=None, no_window=False, logdir=None, xml_output=None,
-                 repeat=0, testvars=None, tree=None, type=None,
+                 logger=None, no_window=False, logdir=None, logcat_stdout=False,
+                 xml_output=None, repeat=0, testvars=None, tree=None, type=None,
                  device_serial=None, symbols_path=None, timeout=None,
                  shuffle=False, shuffle_seed=random.randint(0, sys.maxint),
                  sdcard=None, this_chunk=1, total_chunks=1, sources=None,
-                 server_root=None, gecko_log=None, **kwargs):
+                 server_root=None, gecko_log=None, result_callbacks=None,
+                 adb_host=None, adb_port=None, **kwargs):
         self.address = address
         self.emulator = emulator
         self.emulator_binary = emulator_binary
@@ -458,9 +475,9 @@ class BaseMarionetteTestRunner(object):
         self.httpd = None
         self.marionette = None
         self.logdir = logdir
+        self.logcat_stdout = logcat_stdout
         self.xml_output = xml_output
         self.repeat = repeat
-        self.testvars = {}
         self.test_kwargs = kwargs
         self.tree = tree
         self.type = type
@@ -481,18 +498,51 @@ class BaseMarionetteTestRunner(object):
         self.mixin_run_tests = []
         self.manifest_skipped_tests = []
         self.tests = []
+        self.result_callbacks = result_callbacks if result_callbacks is not None else []
+        self._adb_host = adb_host
+        self._adb_port = adb_port
 
-        if testvars:
-            if not os.path.exists(testvars):
-                raise IOError('--testvars file does not exist')
+        def gather_debug(test, status):
+            rv = {}
+            marionette = test._marionette_weakref()
 
-            try:
-                with open(testvars) as f:
-                    self.testvars = json.loads(f.read())
-            except ValueError as e:
-                json_path = os.path.abspath(testvars)
-                raise Exception("JSON file (%s) is not properly "
-                                "formatted: %s" % (json_path, e.message))
+            # In the event we're gathering debug without starting a session, skip marionette commands
+            if marionette.session is not None:
+                try:
+                    marionette.set_context(marionette.CONTEXT_CHROME)
+                    rv['screenshot'] = marionette.screenshot()
+                    marionette.set_context(marionette.CONTEXT_CONTENT)
+                    rv['source'] = marionette.page_source
+                except:
+                    logger = get_default_logger()
+                    logger.warning('Failed to gather test failure debug.', exc_info=True)
+            return rv
+
+        self.result_callbacks.append(gather_debug)
+
+        def update(d, u):
+            """ Update a dictionary that may contain nested dictionaries. """
+            for k, v in u.iteritems():
+                o = d.get(k, {})
+                if isinstance(v, dict) and isinstance(o, dict):
+                    d[k] = update(d.get(k, {}), v)
+                else:
+                    d[k] = u[k]
+            return d
+
+        self.testvars = {}
+        if testvars is not None:
+            for path in list(testvars):
+                if not os.path.exists(path):
+                    raise IOError('--testvars file %s does not exist' % path)
+                try:
+                    with open(path) as f:
+                        self.testvars = update(self.testvars,
+                                               json.loads(f.read()))
+                except ValueError as e:
+                    raise Exception("JSON file (%s) is not properly "
+                                    "formatted: %s" % (os.path.abspath(path),
+                                                       e.message))
 
         # set up test handlers
         self.test_handlers = []
@@ -561,6 +611,8 @@ class BaseMarionetteTestRunner(object):
             'device_serial': self.device_serial,
             'symbols_path': self.symbols_path,
             'timeout': self.timeout,
+            'adb_host': self._adb_host,
+            'adb_port': self._adb_port,
         }
         if self.bin:
             kwargs.update({
@@ -610,6 +662,53 @@ class BaseMarionetteTestRunner(object):
     def start_marionette(self):
         self.marionette = Marionette(**self._build_kwargs())
 
+    def launch_test_container(self):
+        if self.marionette.session is None:
+            self.marionette.start_session()
+        self.marionette.set_context(self.marionette.CONTEXT_CONTENT)
+
+        result = self.marionette.execute_async_script("""
+if((navigator.mozSettings == undefined) || (navigator.mozSettings == null) || (navigator.mozApps == undefined) || (navigator.mozApps == null)) {
+    marionetteScriptFinished(false);
+    return;
+}
+let setReq = navigator.mozSettings.createLock().set({'lockscreen.enabled': false});
+setReq.onsuccess = function() {
+    let appName = 'Test Container';
+    let activeApp = window.wrappedJSObject.Service.currentApp;
+
+    // if the Test Container is already open then do nothing
+    if(activeApp.name === appName){
+        marionetteScriptFinished(true);
+    }
+
+    let appsReq = navigator.mozApps.mgmt.getAll();
+    appsReq.onsuccess = function() {
+        let apps = appsReq.result;
+        for (let i = 0; i < apps.length; i++) {
+            let app = apps[i];
+            if (app.manifest.name === appName) {
+                app.launch();
+                window.addEventListener('appopen', function apploadtime(){
+                    window.removeEventListener('appopen', apploadtime);
+                    marionetteScriptFinished(true);
+                });
+                return;
+            }
+        }
+        marionetteScriptFinished(false);
+    }
+    appsReq.onerror = function() {
+        marionetteScriptFinished(false);
+    }
+}
+setReq.onerror = function() {
+    marionetteScriptFinished(false);
+}""", script_timeout=60000)
+
+        if not result:
+            raise Exception("Could not launch test container app")
+
     def run_tests(self, tests):
         self.reset_test_stats()
         self.start_time = time.time()
@@ -634,7 +733,21 @@ class BaseMarionetteTestRunner(object):
         for test in tests:
             self.add_test(test)
 
-        self.logger.suite_start(self.tests)
+        version_info = mozversion.get_version(binary=self.bin,
+                                              sources=self.sources,
+                                              dm_type=os.environ.get('DM_TRANS', 'adb'),
+                                              device_serial=self.device_serial,
+                                              adb_host=self.marionette.adb_host,
+                                              adb_port=self.marionette.adb_port)
+
+        device_info = None
+        if self.capabilities['device'] != 'desktop' and self.capabilities['browserName'] == 'B2G':
+            dm = get_dm(self.marionette)
+            device_info = dm.getInfo()
+
+        self.logger.suite_start(self.tests,
+                                version_info=version_info,
+                                device_info=device_info)
 
         for test in self.manifest_skipped_tests:
             name = os.path.basename(test['path'])
@@ -696,7 +809,7 @@ class BaseMarionetteTestRunner(object):
 
         self.logger.suite_end()
 
-    def add_test(self, test, expected='pass', oop=None):
+    def add_test(self, test, expected='pass', test_container=None):
         filepath = os.path.abspath(test)
 
         if os.path.isdir(filepath):
@@ -719,8 +832,7 @@ class BaseMarionetteTestRunner(object):
                 else:
                     testargs.update({ atype: 'true' })
 
-        # testarg_oop = either None, 'true' or 'false'.
-        testarg_oop = testargs.get('oop')
+        testarg_b2g = bool(testargs.get('b2g'))
 
         file_ext = os.path.splitext(os.path.split(filepath)[-1])[1]
 
@@ -740,11 +852,6 @@ class BaseMarionetteTestRunner(object):
                 else:
                     unfiltered_tests.append(test)
 
-            # Don't filter tests with "oop" flag because manifest parser can't
-            # handle it well.
-            if testarg_oop is not None:
-                del testargs['oop']
-
             target_tests = manifest.get(tests=unfiltered_tests, **testargs)
             for test in unfiltered_tests:
                 if test['path'] not in [x['path'] for x in target_tests]:
@@ -755,54 +862,25 @@ class BaseMarionetteTestRunner(object):
                 if not os.path.exists(i["path"]):
                     raise IOError("test file: %s does not exist" % i["path"])
 
-                # manifest_oop is either 'false', 'true' or 'both'.  Anything
-                # else implies 'false'.
-                manifest_oop = i.get('oop', 'false')
-
-                # We only add an oop test when following conditions are met:
-                # 1) It's written by javascript because we have only
-                #    MarionetteJSTestCase that supports oop mode.
-                # 2) we're running with "--type=+oop" or no "--type=-oop", which
-                #    follows testarg_oop is either None or 'true' and must not
-                #    be 'false'.
-                # 3) When no "--type=[+-]oop" is applied, all active tests are
-                #    included in target_tests, so we must filter out those
-                #    really capable of running in oop mode. Besides, oop tests
-                #    must be explicitly specified for backward compatibility. So
-                #    test manifest_oop equals to either 'both' or 'true'.
                 file_ext = os.path.splitext(os.path.split(i['path'])[-1])[-1]
-                if (file_ext == '.js' and
-                    testarg_oop != 'false' and
-                    (manifest_oop == 'both' or manifest_oop == 'true')):
-                    self.add_test(i["path"], i["expected"], True)
+                test_container = None
+                if i.get('test_container') and testarg_b2g:
+                    if i.get('test_container') == "true":
+                        test_container = True
+                    elif i.get('test_container') == "false":
+                        test_container = False
 
-                # We only add an in-process test when following conditions are
-                # met:
-                # 1) we're running with "--type=-oop" or no "--type=+oop", which
-                #    follows testarg_oop is either None or 'false' and must not
-                #    be 'true'.
-                # 2) When no "--type=[+-]oop" is applied, all active tests are
-                #    included in target_tests, so we must filter out those
-                #    really capable of running in in-process mode.
-                if (testarg_oop != 'true' and
-                    (manifest_oop == 'both' or manifest_oop != 'true')):
-                    self.add_test(i["path"], i["expected"], False)
+                self.add_test(i["path"], i["expected"], test_container)
             return
 
-        if oop is None:
-            # This test is added by directory enumeration or directly specified
-            # in argument list.  We have no manifest information here so we just
-            # respect the "--type=[+-]oop" argument here.
-            oop = file_ext == '.js' and testarg_oop == 'true'
+        self.tests.append({'filepath': filepath, 'expected': expected, 'test_container': test_container})
 
-        self.tests.append({'filepath': filepath, 'expected': expected, 'oop': oop})
-
-    def run_test(self, filepath, expected, oop):
+    def run_test(self, filepath, expected, test_container):
 
         testloader = unittest.TestLoader()
         suite = unittest.TestSuite()
         self.test_kwargs['expected'] = expected
-        self.test_kwargs['oop'] = oop
+        self.test_kwargs['test_container'] = test_container
         mod_name = os.path.splitext(os.path.split(filepath)[-1])[0]
         for handler in self.test_handlers:
             if handler.match(os.path.basename(filepath)):
@@ -818,7 +896,13 @@ class BaseMarionetteTestRunner(object):
         if suite.countTestCases():
             runner = self.textrunnerclass(logger=self.logger,
                                           marionette=self.marionette,
-                                          capabilities=self.capabilities)
+                                          capabilities=self.capabilities,
+                                          logcat_stdout=self.logcat_stdout,
+                                          result_callbacks=self.result_callbacks)
+
+            if test_container:
+                self.launch_test_container()
+
             results = runner.run(suite)
             self.results.append(results)
 
@@ -843,12 +927,14 @@ class BaseMarionetteTestRunner(object):
             random.shuffle(tests)
 
         for test in tests:
-            self.run_test(test['filepath'], test['expected'], test['oop'])
+            self.run_test(test['filepath'], test['expected'], test['test_container'])
             if self.marionette.check_for_crash():
                 break
 
     def run_test_sets(self):
-        if self.total_chunks > len(self.tests):
+        if len(self.tests) < 1:
+            raise Exception('There are no tests to run.')
+        elif self.total_chunks > len(self.tests):
             raise ValueError('Total number of chunks must be between 1 and %d.' % len(self.tests))
         if self.total_chunks > 1:
             chunks = [[] for i in range(self.total_chunks)]
@@ -862,16 +948,15 @@ class BaseMarionetteTestRunner(object):
                                                len(self.tests)))
             self.tests = chunks[self.this_chunk - 1]
 
-        oop_tests = [x for x in self.tests if x.get('oop')]
-        self.run_test_set(oop_tests)
-
-        in_process_tests = [x for x in self.tests if not x.get('oop')]
-        self.run_test_set(in_process_tests)
+        self.run_test_set(self.tests)
 
     def cleanup(self):
         if self.httpd:
             self.httpd.stop()
 
+        if self.marionette:
+            self.marionette.cleanup()
+
     __del__ = cleanup
 
     def generate_xml(self, results_list):
